import { _decorator, Component, Vec3, Quat, Node, systemEvent, SystemEvent, EventMouse, AnimationComponent,SkeletalAnimationComponent } from "cc";
const { ccclass, property } = _decorator;

@ccclass("PlayerController")
export class PlayerController extends Component {
    /* class member could be defined like this */
    // dummy = '';

    @property({type:Node})
    mainCarmera: Node = null;
    @property({type: AnimationComponent})
    public BodyAnim: AnimationComponent = null;

    @property({type: SkeletalAnimationComponent})
    public CocosAnim: SkeletalAnimationComponent = null;
    /* use `property` decorator if your want the member to be serializable */
    // @property
    // serializableDummy = 0;
    private _startJump: boolean = false;
    private _jumpStep: number = 0;
    private _curJumpTime: number = 0;
    private _jumpTime: number = 0.1;
    private _curJumpSpeed: number = 0;
    private _curPos: Vec3 = cc.v3();
    private _deltaPos: Vec3 = cc.v3(0, 0, 0);
    private _targetPos: Vec3 = cc.v3();
    private _isMoving = false;    
    private _curMoveIndex = 0;

    private _curRotateX:number = 0;
    private _curRotateY:number = 0;

    start () {
        // Your initialization goes here.
        // systemEvent.on(SystemEvent.EventType.MOUSE_UP, this.onMouseUp, this);
        cc.log("now start:", this.mainCarmera.getPosition(), this.mainCarmera.getRotation())
    }

    setInputActive(active: boolean) {
        if (active) {
            systemEvent.on(SystemEvent.EventType.MOUSE_UP, this.onMouseUp, this);
            systemEvent.on(SystemEvent.EventType.KEY_DOWN, this.onKeyDown, this);
        } else {
            systemEvent.off(SystemEvent.EventType.MOUSE_UP, this.onMouseUp, this);
            systemEvent.off(SystemEvent.EventType.KEY_DOWN, this.onKeyDown, this);
        }
    }

    onMouseUp(event: EventMouse) {
        if (event.getButton() === 0) {
            this.jumpByStep(1);
        } else if (event.getButton() === 2) {
            this.jumpByStep(2);
        }

    }

    onKeyDown(event) {
        cc.log(event.keyCode)
        switch(event.keyCode) {
            case 65:
                this.rotation(0, -2);
                break;
            case 68:
                this.rotation(0, 2);
                break;
            case 87:
                this.rotation(2, 0);
                break;
            case 83:
                this.rotation(-2, 0);
                break;
        }
    }

    jumpByStep(step: number) {
        if (this._isMoving) {
            return;
        }
        this._startJump = true;
        this._jumpStep = step;
        this._curJumpTime = 0;
        this._curJumpSpeed = this._jumpStep / this._jumpTime;
        this._curMoveIndex += step;
        this.node.getPosition(this._curPos);
        Vec3.add(this._targetPos, this._curPos, cc.v3(this._jumpStep, 0, 0));
        this._isMoving = true;

        this.CocosAnim.getState('cocos_anim_jump').speed = 3.5; //跳跃动画时间比较长，这里加速播放
        this.CocosAnim.play('cocos_anim_jump'); //播放跳跃动画
        if (step === 1) {
            //this.BodyAnim.play('oneStep');
        } else if (step === 2) {
            this.BodyAnim.play();
        }
        this.node.emit('JumpStart');

    }

    onOnceJumpEnd() {
        this._isMoving = false;
        this.node.emit('JumpEnd', this._curMoveIndex);
        this.CocosAnim.play('cocos_anim_idle');
    }

    rotation(x:number, y:number) {
        this._curRotateX += x;
        this._curRotateY += y;
        let p = cc.v3(0, 0, 17);
        let a = this._curRotateX  * Math.PI / 180;
        let b = this._curRotateY  * Math.PI / 180;
        let p1 = [p.x, p.y * Math.cos(a) - p.z * Math.sin(a),  p.y * Math.sin(a) + p.z * Math.cos(a)];
        let p2 = cc.v3(p1[0] * Math.cos(b) - p1[2] * Math.sin(b), p1[1], p1[2] * Math.cos(b) + p1[0] * Math.sin(b));


        this.mainCarmera.setPosition(p2);
        let rot = cc.quat(0, 0, 0, 0);
        Quat.fromEuler(rot, this._curRotateX, this._curRotateY * -1, 0);
        cc.log("cal rotation:", rot)
        this.mainCarmera.setRotation(rot);
        cc.log("now p2:", this.mainCarmera.getPosition(), this.mainCarmera.getRotation(), this._curRotateX, this._curRotateY)
        // this.mainCarmera.setPosition(p2);
        /*
        绕X轴旋转a度
        x1=x;
        y1=y*cos（a）-z*sin（a）;
        z1=z*cos（a）+y*sin（a）;
        绕Y轴旋转a度 
        x1=x*cos（a）-z*sin（a）;
        y1=y;
        z1=z*cos（a）+x*sin（a）;
        */
    }
    

    reset() {
        this._curMoveIndex = 0;
    }

    update (deltaTime: number) {
        if (this._startJump) {
            this._curJumpTime += deltaTime;
            if (this._curJumpTime > this._jumpTime) {
                // end
                this.node.setPosition(this._targetPos);
                this._startJump = false;
                this.onOnceJumpEnd();
            } else {
                // tween
                this.node.getPosition(this._curPos);
                this._deltaPos.x = this._curJumpSpeed * deltaTime;
                Vec3.add(this._curPos, this._curPos, this._deltaPos);
                this.node.setPosition(this._curPos);
            }
        }
    }
}
